#!/usr/bin/env Rscript

cargs <- commandArgs(T)
lentable <- cargs[1]
out <- cargs[2]
pare <- as.numeric(cargs[3])
pcut <- as.numeric(cargs[4])
minclust <- as.numeric(cargs[5])
minclustprop <- as.numeric(cargs[6])
minintfid <- as.numeric(cargs[7])

if (length(cargs) != 7) {
	message("\nUsage: Rscript histoneCalc.R lenTable.txt out_prefix pared_values min_p_val min_clusters min_prop_clusters min_period_coef")
	message("Input files are read length tables as in Supp Data S2, \"lenTables\"")
	message("For plots as in Kistler et al 2017 BioRxiv \"A new model for ancient DNA decay...\", try \"Rscript histoneCalc.R infile.lenTable.txt out_prefix 4 0.001 2 0.0 0.2\"")
	message("For parameter details, see the Supp Methods in the above preprint\n")
	
	message("Note: This script produces NO OUTPUT FILES if histone periodicity cannot be estimated from the input length table\n")
	quit();
}

p <- paste(out,".histones.pdf",sep="")
outfile <- paste(out,".histoneStat.txt",sep="")

minval <- 0.002
r <- read.table(lentable)
r[,2] <-r[,2]/sum(r[,2])

x <- r[,1]
y <- r[,2]
ping <- length(y)
for (f in 1:50) {
	i <- length(y)
	j <- i-1
	if (y[j]<y[i]) {
		y <- y[1:j]
		x <- x[1:j]
	}
}

for (i in 1:1000) {
	if (y[length(y)] < minval) {
		y <- y[1:(length(y)-1)]
		x <- x[1:(length(x)-1)]
	}
	else {
		break
	}
}

r <- cbind(x,y)
for (i in 1:100) {
	if (r[1,2] < minval) {
		r <- r[2:length(r[,1]),]
	}
	else {
		break
	}
}

x <- r[,1]
y <- r[,2]

for (f in 2:(length(y)-1)) {
	ping <- y[f]/((y[f-1]+y[f+1])/2)
	if (ping>=1.5) {
		p1 <- y[f-1]
		p2 <- y[f+1]
		if (p1 > p2) {
			y[f] <- p1-(diff(c(p1,p2)))
		}
		else if (p2 > p1) {
			y[f] <- p2-(diff(c(p1,p2)))
		}
	}
}

r <- cbind(x,y)


highclip <- ping - length(r[,1])
ping <- length(r[,1])

lowclip <- ping - length(r[,1])

l <- lowess(r[,1],r[,2],f=20/diff(range(r[,1])))
dif <- r[,2]/l$y

dif <- dif[(pare+1):(length(dif)-pare)]

err <- c()
#err2 <- c()
h <- c()
is <- c()
fs <- c()
Ns <- c()
ks <- c()
for (i in 1:(length(dif)-11)) {
	for (f in 8:12) {
		t <- i+(f-1)
		set <- dif[i:t]
		len <- 1:length(set)
		len <- len-median(len)
		get <- try(b <- summary(nls(set~k+abs(N)*(len**2), start=list(k=1,N=.1))),silent=TRUE)
		if (class(get)=="summary.nls") {
			stat <- b$coefficients[8]
			N <- b$coefficients[2]
			e <- b$coefficients[4]
			#e2 <- sum(log(abs(b$residuals)))
			h <- c(h, stat)
			is <- c(is, i)
			fs <- c(fs, f)
			Ns <- c(Ns,N)
			err <- c(err, e)
			#err2 <- c(err2, e2)
		}
	}
}

df <- data.frame(h,is,fs,Ns,err)
names(df) <- c("p-value", "starting_pos","length","N-value","N-stderr")
df <- df[df[,1]<=pcut,]

library(Ckmeans.1d.dp)
lim <- length(unique(df[,2]))-1
c <- Ckmeans.1d.dp(df[,2],k=c(1,lim))

max <- as.integer((length(dif)/10))
clusterProp <- length(c$centers)/max
jump <- diff(c$centers)
ping <- 0
int <- c()
for (i in jump) {
	if (i >= 8 & i <= 12) {
		ping <- ping+1
		int <- c(int,i)
	}
	else if (i > 12) {
		for (j in 1:max) {
			if (i/j >= 8 & i/j <=12) {
				ping <- ping + 1/j
				int <- c(int,i/j)
			}
		}
	}
}

intprop <- ping/(length(c$centers)-1)

if (length(c$centers)>=minclust & clusterProp>=minclustprop & intprop>=minintfid) {
	offset <- min(r[,1])+pare
	cent <- c$centers + offset
	pdf(file=p, width=12, height=10)
	par(mfrow=c(2,2))
	plot(r,
		main="Fragment length distribution and fitted lowess curve",
		xlab="Fragment size",
		ylab="Proportion of reads")
	for (i in cent) {
		abline(v=i, lty=2, lwd=.5)
	}

	points(l, pch=19, cex=.35)
	lines(l, lwd=.5)
	difx <- (1:length(dif))+offset
	plot(difx,dif,
		main="Deviation of data from smoothing curve",
		xlab="Fragment size",
		ylab="Observed/Lowess")
	lines(difx, dif)
	for (i in cent) {
		abline(v=i, lty=2, lwd=.5)
	}
	plot(table(df[,2]+offset),
		main="Clusters of significant fits",
		xlab="Fragment size",
		ylab=paste("number significant fits at p<=",pcut))
	plot(density(df[,4]),
		main="Density of signal intensity estimates")
	q <- quantile(df[,4],c(.025,.5,.975))
	abline(v=q[2])
	abline(v=q[1], lty=2)
	abline(v=q[3], lty=2)
	dev.off()

	write.table(df,file=outfile,quote=F, row.names=F)
	write.table(q,file=outfile,quote=F,append=T)
}

#else {
#	offset <- min(r[,1])+pare
#	cent <- c$centers + offset
#	pdf(file=p, width=12, height=5)
#	par(mfrow=c(1,2))
#	plot(r,
#		main="Fragment length distribution and fitted lowess curve",
#		xlab="Fragment size",
#		ylab="Proportion of reads")
#	for (i in cent) {
#		abline(v=i, lty=2, lwd=.5)
#	}
#
#	points(l, pch=19, cex=.35)
#	lines(l, lwd=.5)
#	difx <- (1:length(dif))+offset
#	plot(difx,dif,
#		main="Deviation of data from smoothing curve",
#		xlab="Fragment size",
#		ylab="Observed/Lowess")
#}
